//---------------------------------------------------------------------------
#ifndef gamestateH
#define gamestateH

/** Classes to manage the various gamestates.
*/

#ifdef __linux
#include <SDL/SDL.h>
#else
#include <sdl.h>
#endif

#include <vector>
using std::vector;
#include <string>
using std::string;

#include "game.h"

class Image;
class ImageButton;
class Button;
class PanelPage;
class ChooseGameTypePanel;
class ChooseDifficultyPanel;
class ChooseMenPanel;
class GamePanel;
class Sector;
class Army;
class Soldier;
class Building;
class Vector;

const int offset_map_x_c = 8;
const int offset_map_y_c = 16;

const int offset_land_x_c = 96;
const int offset_land_y_c = 50;

const int offset_openpitmine_x_c = 32;
const int offset_openpitmine_y_c = 32;

const int land_width_c =  300 - offset_land_x_c;
const int land_height_c = 210 - offset_land_y_c;

//const int max_soldiers_in_sector_c = 1000;

class Feature {
protected:
	const Image *image;
	int xpos, ypos;
public:
	Feature(const Image *image,int xpos,int ypos) {
		this->image = image;
		this->xpos = xpos;
		this->ypos = ypos;
	}

	const Image *getImage() const {
		return this->image;
	}
	int getX() const {
		return this->xpos;
	}
	int getY() const {
		return this->ypos;
	}
};

class TimedEffect {
protected:
	int timeset;
	void (*func_finish) ();
public:
	TimedEffect();
	TimedEffect(int delay, void (*func_finish)());
	virtual ~TimedEffect() {
		if( func_finish != NULL ) {
			// we need to make a copy, incase this class is
			// destroyed from within the called function
			void (*temp_func_finish) ();
			temp_func_finish = func_finish;
			func_finish = NULL;
			temp_func_finish();
		}
	}
	virtual bool render() const {
		return false;
	}
};

class AmmoEffect : public TimedEffect {
	int gametimeset;
	int epoch;
	AmmoDirection dir;
	int xpos, ypos;
public:
	AmmoEffect(int epoch, AmmoDirection dir, int xpos, int ypos);
	virtual bool render() const;
};

class FadeEffect : public TimedEffect {
	Image *image;
	bool white;
	bool out;
public:
	FadeEffect(bool white,bool out,int delay, void (*func_finish)());
	virtual ~FadeEffect();
	virtual bool render() const;
};

class FlashingSquare : public TimedEffect {
	int xpos, ypos;
public:
	FlashingSquare(int xpos, int ypos) : TimedEffect() {
		this->xpos = xpos;
		this->ypos = ypos;
	}

	virtual bool render() const;
};

class AnimationEffect : public TimedEffect {
	Image **images;
	int n_images;
	int xpos, ypos;
	int time_per_frame;
	bool dir;
public:
	AnimationEffect(int xpos, int ypos,Image **images, int n_images,int time_per_frame,bool dir) : TimedEffect() {
		this->images = images;
		this->n_images = n_images;
		this->xpos = xpos;
		this->ypos = ypos;
		this->time_per_frame = time_per_frame;
		this->dir = dir;
	}
	virtual bool render() const;
};

class TextEffect : public TimedEffect {
	int xpos, ypos;
	string text;
	int duration;
public:
	TextEffect(string text, int xpos, int ypos, int duration) : TimedEffect(), text(text), xpos(xpos), ypos(ypos), duration(duration) {
	}
	virtual bool render() const;
};

/*class GameState {
FadeEffect *fade;
FadeEffect *whitefade;
Vector *effects;
Vector *ammo_effects;
ImageButton *speed_button;
PanelPage *screen_page;
PanelPage *land_panel;
ChooseGameTypePanel *choosegametypePanel;
ChooseMenPanel *choosemenPanel;
GamePanel *gamePanel;
int flag_frame_step;
int defenders_last_frame_update;
int soldiers_last_time_moved;
int soldiers_last_time_turned;
Army *selected_army;
int n_soldiers[n_players_c];
Vector *soldiers[n_players_c];

bool openPitMine();
bool validSoldierLocation(int epoch,int xpos,int ypos);
bool buildingMouseClick(int s_m_x,int s_m_y,bool m_left,bool m_right,Building *building);
void moveTo(int map_x,int map_y);
void blueEffect(int xpos,int ypos,bool dir);

public:
PanelPage *map_panels[map_width_c][map_height_c];
int n_deaths[n_players_c][n_epochs_c+1];

GameState();
~GameState();

ChooseGameTypePanel *getChooseGameTypePanel();
ChooseMenPanel *getChooseMenPanel();
GamePanel *getGamePanel();
void setFlashingSquare(int xpos,int ypos);
void fadeScreen(bool out, int delay, void (*func_finish)());
void whiteFlash();
void reset();
void addBuilding(Building *building);
void draw();
void update();
void mouseClick(int m_x,int m_y,Uint8 m_b);
void refreshSoldiers(bool flash);
//void clearSoldiers();
void deathEffect(int xpos,int ypos);
void refreshButtons();
};*/

class GameState {
protected:
	FadeEffect *fade;
	FadeEffect *whitefade;
	//Vector *effects;
	//vector<TimedEffect *> *effects;
	PanelPage *screen_page;
	Image *mouse_image;
	int mouse_off_x, mouse_off_y;

public:
	GameState();
	virtual ~GameState();

	virtual void reset();
	virtual void draw();
	virtual void update() {};
	virtual void mouseClick(int m_x,int m_y,Uint8 m_b);

	void fadeScreen(bool out, int delay, void (*func_finish)());
	void whiteFlash();
	/*void addEffect(TimedEffect *effect) {
		this->effects->push_back(effect);
	}*/
	virtual void addTextEffect(TextEffect *effect) {
		// only implemented for PlayingGameState
	}
};

class ChooseGameTypeGameState : public GameState {
	ChooseGameTypePanel *choosegametypePanel;

public:
	ChooseGameTypeGameState();
	virtual ~ChooseGameTypeGameState();

	virtual void reset();
	virtual void draw();
	virtual void mouseClick(int m_x,int m_y,Uint8 m_b);

	ChooseGameTypePanel *getChooseGameTypePanel();
};

class ChooseDifficultyGameState : public GameState {
	ChooseDifficultyPanel *choosedifficultyPanel;

public:
	ChooseDifficultyGameState();
	virtual ~ChooseDifficultyGameState();

	virtual void reset();
	virtual void draw();
	virtual void mouseClick(int m_x,int m_y,Uint8 m_b);

	ChooseDifficultyPanel *getChooseDifficultyPanel();
};

class ChoosePlayerGameState : public GameState {
public:
	ChoosePlayerGameState();
	virtual ~ChoosePlayerGameState();

	virtual void draw();
	virtual void mouseClick(int m_x,int m_y,Uint8 m_b);
};

class PlaceMenGameState : public GameState {
	ChooseMenPanel *choosemenPanel;
	int off_x;
	PanelPage *map_panels[map_width_c][map_height_c];
	int start_map_x, start_map_y;

public:
	PlaceMenGameState();
	virtual ~PlaceMenGameState();

	virtual void reset();
	virtual void draw();
	virtual void mouseClick(int m_x,int m_y,Uint8 m_b);

	ChooseMenPanel *getChooseMenPanel();
	int getOffX() const {
		return this->off_x;
	}
	const PanelPage *getMapPanel(int x, int y) const;
	PanelPage *getMapPanel(int x, int y);
	int getStartMapX() const {
		return this->start_map_x;
	}
	int getStartMapY() const {
		return this->start_map_y;
	}
};

class PlayingGameState : public GameState {
	GamePanel *gamePanel;
	ImageButton *speed_button;
	ImageButton *shield_buttons[n_players_c];
	ImageButton *shield_blank_button;
	PanelPage *shield_number_panels[n_players_c];
	PanelPage *land_panel;
	int flag_frame_step;
	int defenders_last_frame_update;
	int soldiers_last_time_moved;
	int soldiers_last_time_turned;
	int air_last_time_moved;
	Army *selected_army;
	//int n_soldiers[n_players_c];
	//Vector *soldiers[n_players_c];
	vector<Soldier *> *soldiers[n_players_c];
	vector<TimedEffect *> *effects;
	//Vector *ammo_effects;
	vector<TimedEffect *> *ammo_effects;
	TextEffect *text_effect;
	enum MapDisplay {
		MAPDISPLAY_MAP = 0,
		MAPDISPLAY_UNITS = 1
	};
	MapDisplay map_display;
	int player_asking_alliance;
	PanelPage *map_panels[map_width_c][map_height_c];
	Button *alliance_yes;
	Button *alliance_no;
	int n_deaths[n_players_c][n_epochs_c+1];

	bool openPitMine();
	bool validSoldierLocation(int epoch,int xpos,int ypos);
	bool buildingMouseClick(int s_m_x,int s_m_y,bool m_left,bool m_right,Building *building);
	void moveTo(int map_x,int map_y);
	void blueEffect(int xpos,int ypos,bool dir);
	void refreshShieldNumberPanels();
	void setupMapGUI();
public:

	PlayingGameState();
	virtual ~PlayingGameState();

	virtual void reset();
	virtual void draw();
	virtual void update();
	virtual void mouseClick(int m_x,int m_y,Uint8 m_b);

	GamePanel *getGamePanel();
	void resetShieldButtons();
	void setFlashingSquare(int xpos,int ypos);
	void addBuilding(Building *building);
	void refreshSoldiers(bool flash);
	void deathEffect(int xpos,int ypos);
	void addEffect(TimedEffect *effect) {
		this->effects->push_back(effect);
	}
	virtual void addTextEffect(TextEffect *effect) {
		if( text_effect != NULL ) {
			delete text_effect;
		}
		text_effect = effect;
	}
	void refreshButtons();
	const Army *getSelectedArmy() const {
		return this->selected_army;
	}
	Army *getSelectedArmy() {
		return this->selected_army;
	}
	void clearSelectedArmy() {
		this->selected_army = NULL;
	}
	bool canRequestAlliance(int player,int i) const;
	void requestAlliance(int player,int i,bool human);
	void makeAlliance(int player,int i);
	const int getPlayerAskingAlliance() const {
		return this->player_asking_alliance;
	}
	void cancelPlayerAskingAlliance();
	void registerDeath(int player, int epoch) {
		n_deaths[player][epoch]++;
	}
};

class EndIslandGameState : public GameState {
public:
	EndIslandGameState() {
	}
	virtual ~EndIslandGameState() {
	}

	virtual void reset();
	virtual void draw();
	virtual void mouseClick(int m_x,int m_y,Uint8 m_b);
};

class GameCompleteGameState : public GameState {
public:
	GameCompleteGameState() {
	}
	virtual ~GameCompleteGameState() {
	}

	virtual void reset();
	virtual void draw();
	virtual void mouseClick(int m_x,int m_y,Uint8 m_b);
};

//---------------------------------------------------------------------------
#endif
